/*
    Парус 8 - Панели мониторинга - ТОиР - Прогнозирование технического состояния
    Закладка администрирования: кастомные хуки
*/

//---------------------
//Подключение библиотек
//---------------------

import { useState, useEffect, useContext } from "react"; //Классы React
import { BackEndСtx } from "../../context/backend"; //Контекст взаимодействия с сервером

//---------
//Константы
//---------

//Рег. номер выборки данных по умолчанию
const DS_RN_DEFAULT = -1;

//Наименование выборки данных по умолчанию
const DS_NAME_DEFAULT = "Не указана";

//Выборка данных по умолчанию
const DS_DEFAULT = { NRN: DS_RN_DEFAULT, SNAME: DS_NAME_DEFAULT };

//-----------
//Тело модуля
//-----------

//Загрузка списка выборок данных
const useEquipDataSelectionList = refresh => {
    //Собственное состояние - флаг загрузки
    const [isLoading, setLoading] = useState(false);

    //Собственное состояние - данные
    const [data, setData] = useState([DS_DEFAULT]);

    //Подключение к контексту взаимодействия с сервером
    const { executeStored } = useContext(BackEndСtx);

    //Загрузка данных при изменении зависимостей
    useEffect(() => {
        const loadData = async () => {
            try {
                setLoading(true);
                const data = await executeStored({
                    stored: "UDO_PKG_EQUIPDS.LIST",
                    respArg: "COUT",
                    isArray: name => ["XDS"].includes(name),
                    attributeValueProcessor: (name, val) => (["SNAME"].includes(name) ? undefined : val),
                    loader: false
                });
                setData([DS_DEFAULT, ...(data?.XDS ? data.XDS : [])]);
            } finally {
                setLoading(false);
            }
        };
        loadData();
    }, [refresh, executeStored]);

    //Вернём данные
    return { equipDataSelectionList: data, equipDataSelectionListIsLoading: isLoading };
};

//Загрузка классов оборудования выбороки данных
const useEquipDataSelectionClassMachineList = (dataSelection, refresh) => {
    //Собственное состояние - флаг загрузки
    const [isLoading, setLoading] = useState(false);

    //Собственное состояние - данные
    const [data, setData] = useState([]);

    //Подключение к контексту взаимодействия с сервером
    const { executeStored } = useContext(BackEndСtx);

    //Загрузка данных при изменении зависимостейuse
    useEffect(() => {
        const loadData = async () => {
            try {
                setLoading(true);
                const data = await executeStored({
                    stored: "UDO_PKG_EQUIPDS.CM_LIST",
                    args: { NEQUIPDS: dataSelection },
                    respArg: "COUT",
                    isArray: name => ["XDSCM"].includes(name),
                    attributeValueProcessor: (name, val) => (["SCODE"].includes(name) ? undefined : val),
                    loader: false
                });
                setData(data?.XDSCM || []);
            } finally {
                setLoading(false);
            }
        };
        if (dataSelection && dataSelection != DS_RN_DEFAULT) loadData();
    }, [dataSelection, refresh, executeStored]);

    //Вернём данные
    return { equipDataSelectionClassMachineList: data, equipDataSelectionClassMachineListIsLoading: isLoading };
};

//Загрузка карточки класса оборудования
const useEquipDataSelectionClassMachineCard = (classMachine, refresh) => {
    //Собственное состояние - флаг загрузки
    const [isLoading, setLoading] = useState(false);

    //Собственное состояние - данные карточки
    const [data, setData] = useState({});

    //Подключение к контексту взаимодействия с сервером
    const { executeStored } = useContext(BackEndСtx);

    //Загрузка данных при изменении зависимостей
    useEffect(() => {
        const loadData = async () => {
            try {
                setLoading(true);
                const data = await executeStored({
                    stored: "UDO_PKG_EQUIPDS.CM_CARD",
                    args: { NEQUIPDSCM: classMachine },
                    respArg: "COUT",
                    loader: false
                });
                setData(data.XEQUIPDSCM);
            } finally {
                setLoading(false);
            }
        };
        if (classMachine) loadData();
    }, [classMachine, refresh, executeStored]);

    //Вернём данные
    return { equipDataSelectionClassMachineCard: data, equipDataSelectionClassMachineCardIsLoading: isLoading };
};

//Загрузка списка файлов класса оборудования
const useEquipDataSelectionClassMachineFilesList = (classMachine, refresh) => {
    //Собственное состояние - флаг загрузки
    const [isLoading, setLoading] = useState(false);

    //Собственное состояние - таблица данных
    const [data, setData] = useState({
        init: false,
        columnsDef: [],
        rows: []
    });

    //Подключение к контексту взаимодействия с сервером
    const { executeStored } = useContext(BackEndСtx);

    //Загрузка данных при изменении зависимостей
    useEffect(() => {
        const loadData = async () => {
            try {
                setLoading(true);
                const data = await executeStored({
                    stored: "UDO_PKG_EQUIPDS.CMFL_LIST",
                    args: { NEQUIPDSCM: classMachine },
                    respArg: "COUT",
                    loader: false
                });
                setData(pv => ({
                    ...pv,
                    columnsDef: [...data.XCOLUMNS_DEF],
                    rows: [...(data.XROWS || [])],
                    init: true
                }));
            } finally {
                setLoading(false);
            }
        };
        if (classMachine) loadData();
    }, [classMachine, refresh, executeStored]);

    //Вернём данные
    return { equipDataSelectionClassMachineFilesList: data, equipDataSelectionClassMachineFilesListIsLoading: isLoading };
};

//Загрузка списка моделей класса оборудования
const useEquipDataSelectionClassMachineModelsList = (classMachine, refresh) => {
    //Собственное состояние - флаг загрузки
    const [isLoading, setLoading] = useState(false);

    //Собственное состояние - таблица данных
    const [data, setData] = useState({
        init: false,
        columnsDef: [],
        rows: []
    });

    //Подключение к контексту взаимодействия с сервером
    const { executeStored } = useContext(BackEndСtx);

    //Загрузка данных при изменении зависимостей
    useEffect(() => {
        const loadData = async () => {
            try {
                setLoading(true);
                const data = await executeStored({
                    stored: "UDO_PKG_EQUIPDS.CMML_LIST",
                    args: { NEQUIPDSCM: classMachine },
                    respArg: "COUT",
                    loader: false
                });
                setData(pv => ({
                    ...pv,
                    columnsDef: [...data.XCOLUMNS_DEF],
                    rows: [...(data.XROWS || [])],
                    init: true
                }));
            } finally {
                setLoading(false);
            }
        };
        if (classMachine) loadData();
    }, [classMachine, refresh, executeStored]);

    //Вернём данные
    return { equipDataSelectionClassMachineModelsList: data, equipDataSelectionClassMachineModelsListIsLoading: isLoading };
};

//----------------
//Интерфейс модуля
//----------------

export {
    DS_RN_DEFAULT,
    useEquipDataSelectionList,
    useEquipDataSelectionClassMachineList,
    useEquipDataSelectionClassMachineCard,
    useEquipDataSelectionClassMachineFilesList,
    useEquipDataSelectionClassMachineModelsList
};
