/*
    Парус 8 - Панели мониторинга - ТОиР - Прогнозирование технического состояния
    Закладка администрирования: дополнительная разметка и вёрстка клиентских элементов
*/

//---------------------
//Подключение библиотек
//---------------------

import React, { useState, useContext } from "react"; //Классы React
import PropTypes from "prop-types"; //Контроль свойств компонента
import {
    Box,
    Stack,
    FormControl,
    InputLabel,
    Select,
    MenuItem,
    List,
    ListItem,
    ListItemButton,
    ListItemText,
    Card,
    CardContent,
    Typography,
    Button,
    Dialog,
    DialogTitle,
    DialogContent,
    DialogActions,
    IconButton,
    Icon
} from "@mui/material"; //Интерфейсные компоненты
import { useTheme } from "@mui/material/styles"; //Темы оформления
import { ApplicationСtx } from "../../context/application"; //Контекст приложения
import { BUTTONS } from "../../../app.text"; //Текстовые ресурсы и константы
import { APP_BAR_HEIGHT, TABS_HEIGHT, SCROLL_STYLES, formatModelStateValue, IUDFormTextField } from "./eqs_tech_cond_forecast_lyaout"; //Общие вспомогательные компоненты и вёрстка
import { P8PDataGrid, P8P_DATA_GRID_SIZE } from "../../components/p8p_data_grid"; //Таблица данных
import { P8P_DATA_GRID_CONFIG_PROPS } from "../../config_wrapper"; //Подключение компонентов к настройкам приложения
import { DS_RN_DEFAULT } from "./admin_tab_hooks"; //Вспомогательные хуки

//---------
//Константы
//---------

//Высота списка выборок данных
const DS_SELECTOR_HEIGHT = "80px";

//Высота кнопки добавления технического объекта
const CM_ADD_BTN_HEIGHT = "46.5px";

//Структура элемента списка выборок данных оборудования
const EQUIP_DATA_SELECTION_LIST = PropTypes.shape({
    NRN: PropTypes.number.isRequired,
    SNAME: PropTypes.string.isRequired
});

//Структура элемента списка классов оборудования выборки данных
const EQUIP_DATA_SELECTION_CLASS_MACHINE_LIST = PropTypes.shape({
    NRN: PropTypes.number.isRequired,
    SCODE: PropTypes.string.isRequired,
    SNAME: PropTypes.string.isRequired
});

//Стили
const STYLES = {
    EQUIP_DS_LIST_FORM_CONTROL: { minWidth: "300px" },
    EQUIP_DSCM_LIST: {
        minHeight: `calc(100vh - ${APP_BAR_HEIGHT} - ${TABS_HEIGHT} - ${DS_SELECTOR_HEIGHT} - ${CM_ADD_BTN_HEIGHT})`,
        maxHeight: `calc(100vh - ${APP_BAR_HEIGHT} - ${TABS_HEIGHT} - ${DS_SELECTOR_HEIGHT} - ${CM_ADD_BTN_HEIGHT})`,
        overflowX: "hidden",
        overflowY: "scroll",
        paddingLeft: "15px",
        paddingRight: "15px",
        ...SCROLL_STYLES
    },
    EQUIP_DSCM_CARD: {
        minHeight: `calc(100vh - ${APP_BAR_HEIGHT} - ${TABS_HEIGHT} - ${DS_SELECTOR_HEIGHT})`,
        maxHeight: `calc(100vh - ${APP_BAR_HEIGHT} - ${TABS_HEIGHT} - ${DS_SELECTOR_HEIGHT})`,
        overflowY: "auto",
        ...SCROLL_STYLES
    },
    FL_ML_TABLE: {
        height: `200px`,
        ...SCROLL_STYLES
    },
    EXTRA_CAPTION_BUTTON_CONTAINER: (theme, maxWidth) => ({
        borderWidth: "1px",
        borderStyle: "solid",
        borderColor: theme.palette.grey[400],
        maxWidth,
        borderRadius: theme.shape.borderRadius,
        paddingLeft: "10px",
        paddingRight: "10px",
        paddingTop: "2px",
        paddingBottom: "2px"
    }),
    EXTRA_CAPTION_BUTTON_TITLE: { whiteSpace: "nowrap", textOverflow: "ellipsis", overflow: "hidden" }
};

//------------------------------------
//Вспомогательные функции и компоненты
//------------------------------------

//Формирование значения для колонки "Состояние" файла класса оборудования выборки данных
const formatFileStateValue = (theme, value, err) => {
    const [text, icon, color] =
        value == 0
            ? ["Зарегистрирован", "app_registration", null]
            : value == 1
            ? ["Загружается на сервер", "file_upload", theme.palette.warning.main]
            : value == 2
            ? ["Успешно загружен на сервер", "dns", theme.palette.primary.main]
            : value == 3
            ? [`Ошибка загрузки на сервер: ${err}`, "error", theme.palette.error.main]
            : value == 4
            ? ["Загружается во внешнюю систему", "cloud_upload", theme.palette.warning.main]
            : value == 5
            ? ["Успешно передан внешней системе", "cloud_done", theme.palette.success.main]
            : [`Ошибка передачи внешней системе: ${err}`, "error", theme.palette.error.main];
    return (
        <Stack direction="row" gap={0.5} alignItems="center" justifyContent="left" color={color}>
            <Icon title={text}>{icon}</Icon>
            {text}
        </Stack>
    );
};

//Форматирование колонок таблицы файлов класса оборудования выборки данных
const filesListDataCellRender = ({ row, columnDef, theme }) => {
    switch (columnDef.name) {
        case "NSTATUS":
            return {
                cellProps: { align: "left" },
                data: formatFileStateValue(theme, row.NSTATUS, row.SERR)
            };
    }
};

//Форматирование колонок таблицы моделей класса оборудования выборки данных
const modelsListDataCellRender = ({ row, columnDef, theme, onDelete, onSendRq }) => {
    switch (columnDef.name) {
        case "NSTATUS":
            return {
                cellProps: { align: "left" },
                data: formatModelStateValue(theme, row.NSTATUS, row.SERR)
            };
        case "SACTIONS":
            return {
                data: (
                    <Stack spacing={1} direction={"row"} alignItems={"center"}>
                        <IconButton onClick={() => (onDelete ? onDelete(row.NRN) : null)}>
                            <Icon>delete</Icon>
                        </IconButton>
                        {row.NSTATUS == 0 ? <Button onClick={() => (onSendRq ? onSendRq(row.NRN) : null)}>Обучить</Button> : null}
                    </Stack>
                )
            };
    }
};

//Форматирование ячеек заголовка таблиц файлов и моделей класса оборудования выборки данных
const classMachineCardTableHeadCellRender = ({ columnDef }) => {
    switch (columnDef.name) {
        case "NSTATUS":
            return {
                stackProps: { justifyContent: "left" },
                cellProps: { align: "left" }
            };
        case "SACTIONS":
            return { data: " " };
    }
};

//-----------
//Тело модуля
//-----------

//Диалог IU выборки данных
const EquipDataSelectionIU = ({ onOk, onCancel }) => {
    //Собственное состояние - значения формы
    const [values, setValues] = useState({ code: "", name: "" });

    //Отработка воода значения в форму
    const handleValueChanged = (name, value) => {
        setValues(pv => ({ ...pv, [name]: value }));
    };

    //Генерация содержимого
    return (
        <Dialog open={true} onClose={() => (onOk ? onCancel() : null)}>
            <DialogTitle>Выборка данных</DialogTitle>
            <DialogContent>
                <IUDFormTextField elementCode={"code"} elementValue={values.code} labelText={"Мнемокод"} onChange={handleValueChanged} />
                <IUDFormTextField elementCode={"name"} elementValue={values.name} labelText={"Наименование"} onChange={handleValueChanged} />
            </DialogContent>
            <DialogActions>
                <Button onClick={() => (onOk ? onOk(values) : null)}>{BUTTONS.OK}</Button>
                <Button onClick={() => (onOk ? onCancel() : null)}>{BUTTONS.CANCEL}</Button>
            </DialogActions>
        </Dialog>
    );
};

//Контроль свойств - Диалог IU выборки данных
EquipDataSelectionIU.propTypes = {
    onOk: PropTypes.func,
    onCancel: PropTypes.func
};

//Диалог IU класса оборудования выборки данных
const EquipDataSelectionClassMachineIU = ({ onOk, onCancel }) => {
    //Подключение к контексту приложения
    const { pOnlineShowDictionary } = useContext(ApplicationСtx);

    //Собственное состояние - значения формы
    const [values, setValues] = useState({
        eqobjKind: "",
        userprocsData: "",
        exsServiceUpload: "",
        exsServiceFnUpload: "",
        exsServiceSendMd: "",
        exsServiceFnSendMd: "",
        exsServiceSendRq: "",
        exsServiceFnSendRq: ""
    });

    //Отработка воода значения в форму
    const handleValueChanged = (name, value) => setValues(pv => ({ ...pv, [name]: value }));

    //Выбор класса оборудования из словаря
    const selectEqobjKind = (name, callBack) => {
        pOnlineShowDictionary({
            unitCode: "EquipObjectKinds",
            callBack: res => callBack(res.success ? [{ name, value: res.outParameters.out_FULLCODE }] : null)
        });
    };

    //Выбор процедуры формирования из словаря
    const selectUserProcsData = (name, callBack) => {
        pOnlineShowDictionary({
            unitCode: "UserProcedures",
            callBack: res => callBack(res.success ? [{ name, value: res.outParameters.out_CODE }] : null)
        });
    };

    //Выбор функции обмена для выгрузки данных
    const selectServiceFn = (name, callBack) => {
        pOnlineShowDictionary({
            unitCode: "EXSService",
            callBack: res =>
                callBack(
                    res.success
                        ? [
                              { name, value: res.outParameters.out_FN_CODE },
                              {
                                  name:
                                      name == "exsServiceFnUpload"
                                          ? "exsServiceUpload"
                                          : name == "exsServiceFnSendMd"
                                          ? "exsServiceSendMd"
                                          : "exsServiceSendRq",
                                  value: res.outParameters.out_CODE
                              }
                          ]
                        : null
                )
        });
    };

    //Генерация содержимого
    return (
        <Dialog open={true} onClose={() => (onOk ? onCancel() : null)}>
            <DialogTitle>Класс оборудования выборки данных</DialogTitle>
            <DialogContent>
                <IUDFormTextField
                    elementCode={"eqobjKind"}
                    elementValue={values.eqobjKind}
                    labelText={"Класс оборудования"}
                    onChange={handleValueChanged}
                    dictionary={callBack => selectEqobjKind("eqobjKind", callBack)}
                />
                <IUDFormTextField
                    elementCode={"userprocsData"}
                    elementValue={values.userprocsData}
                    labelText={"Процедура формирования"}
                    onChange={handleValueChanged}
                    dictionary={callBack => selectUserProcsData("userprocsData", callBack)}
                />
                <IUDFormTextField
                    elementCode={"exsServiceUpload"}
                    elementValue={values.exsServiceUpload}
                    labelText={"Сервис обмена для выгрузки данных"}
                    onChange={handleValueChanged}
                    disabled
                />
                <IUDFormTextField
                    elementCode={"exsServiceFnUpload"}
                    elementValue={values.exsServiceFnUpload}
                    labelText={"Функция обмена для выгрузки данных"}
                    onChange={handleValueChanged}
                    dictionary={callBack => selectServiceFn("exsServiceFnUpload", callBack)}
                />
                <IUDFormTextField
                    elementCode={"exsServiceSendMd"}
                    elementValue={values.exsServiceSendMd}
                    labelText={"Сервис обмена для передачи внешней системе"}
                    onChange={handleValueChanged}
                    disabled
                />
                <IUDFormTextField
                    elementCode={"exsServiceFnSendMd"}
                    elementValue={values.exsServiceFnSendMd}
                    labelText={"Функция обмена для передачи внешней системе"}
                    onChange={handleValueChanged}
                    dictionary={callBack => selectServiceFn("exsServiceFnSendMd", callBack)}
                />
                <IUDFormTextField
                    elementCode={"exsServiceSendRq"}
                    elementValue={values.exsServiceSendRq}
                    labelText={"Сервис обмена для обработки внешней системой"}
                    onChange={handleValueChanged}
                    disabled
                />
                <IUDFormTextField
                    elementCode={"exsServiceFnSendRq"}
                    elementValue={values.exsServiceFnSendRq}
                    labelText={"Функция обмена для обработки внешней системой"}
                    onChange={handleValueChanged}
                    dictionary={callBack => selectServiceFn("exsServiceFnSendRq", callBack)}
                />
            </DialogContent>
            <DialogActions>
                <Button onClick={() => (onOk ? onOk(values) : null)}>{BUTTONS.OK}</Button>
                <Button onClick={() => (onOk ? onCancel() : null)}>{BUTTONS.CANCEL}</Button>
            </DialogActions>
        </Dialog>
    );
};

//Контроль свойств - Диалог IU класса оборудования выборки данных
EquipDataSelectionClassMachineIU.propTypes = {
    onOk: PropTypes.func,
    onCancel: PropTypes.func
};

//Диалог IU модели класса оборудования выборки данных
const EquipDataSelectionClassMachineModelIU = ({ onOk, onCancel }) => {
    //Собственное состояние - значения формы
    const [values, setValues] = useState({
        task: "RUL",
        precisionP: ""
    });

    //Отработка воода значения в форму
    const handleValueChanged = (name, value) => setValues(pv => ({ ...pv, [name]: value }));

    //Генерация содержимого
    return (
        <Dialog open={true} onClose={() => (onOk ? onCancel() : null)}>
            <DialogTitle>Модель класса оборудования выборки данных</DialogTitle>
            <DialogContent>
                <IUDFormTextField
                    elementCode={"task"}
                    elementValue={values.task}
                    labelText={"Задача"}
                    onChange={handleValueChanged}
                    list={[
                        { value: null, name: "Не указана" },
                        { value: "TCF", name: "TCF - оценка технического состояния (Technical Condition Forecast)" },
                        { value: "RUL", name: "RUL - прогнозирование остаточного ресурса (Remaining Useful Life)" },
                        { value: "FP", name: "FP - Прогнозирование отказа (Failure Predict)" }
                    ]}
                />
                <IUDFormTextField elementCode={"precisionP"} elementValue={values.precisionP} labelText={"Точность"} onChange={handleValueChanged} />
            </DialogContent>
            <DialogActions>
                <Button onClick={() => (onOk ? onOk(values) : null)}>{BUTTONS.OK}</Button>
                <Button onClick={() => (onOk ? onCancel() : null)}>{BUTTONS.CANCEL}</Button>
            </DialogActions>
        </Dialog>
    );
};

//Контроль свойств - Диалог IU модели класса оборудования выборки данных
EquipDataSelectionClassMachineModelIU.propTypes = {
    onOk: PropTypes.func,
    onCancel: PropTypes.func
};

//Список выборок данных оборудования
const EquipDataSelectionList = ({ list, value, onChange }) => {
    //При выборе элемента
    const handleChange = e => (onChange ? onChange(e.target.value) : null);

    //Генерация содержимого
    return (
        <FormControl variant="standard" sx={STYLES.EQUIP_DS_LIST_FORM_CONTROL}>
            <InputLabel id="equipDSlabel">Выборка данных оборудования</InputLabel>
            <Select
                labelId="equipDSlabel"
                id="equipDS"
                label="Выборка данных оборудования"
                value={list.length == 1 ? DS_RN_DEFAULT : value}
                onChange={handleChange}
                displayEmpty
            >
                {list.map((item, i) => (
                    <MenuItem key={i} value={item.NRN}>
                        {item.SNAME}
                    </MenuItem>
                ))}
            </Select>
        </FormControl>
    );
};

//Контроль свойств - Список выборок данных оборудования
EquipDataSelectionList.propTypes = {
    list: PropTypes.arrayOf(EQUIP_DATA_SELECTION_LIST).isRequired,
    value: PropTypes.number.isRequired,
    onChange: PropTypes.func
};

//Список классов оборудования выборки данных
const EquipDataSelectionClassMachineList = ({ list, value, onClick }) => {
    //При щелчке на элементе
    const handleClick = id => (onClick ? onClick(id) : null);

    //Генерация содержимого
    return (
        <List sx={STYLES.EQUIP_DSCM_LIST}>
            {list.map((item, i) => (
                <ListItem disablePadding key={i}>
                    <ListItemButton selected={value == item.NRN} onClick={() => handleClick(item.NRN)}>
                        <ListItemText primary={item.SCODE} secondary={item.SNAME} />
                    </ListItemButton>
                </ListItem>
            ))}
        </List>
    );
};

//Контроль свойств - Список классов оборудования выборки данных
EquipDataSelectionClassMachineList.propTypes = {
    list: PropTypes.arrayOf(EQUIP_DATA_SELECTION_CLASS_MACHINE_LIST).isRequired,
    value: PropTypes.number,
    onClick: PropTypes.func
};

//Карточка класса оборудования выборки данных
const EquipDataSelectionClassMachineCard = ({
    card,
    filesList,
    modelsList,
    onCardDelete,
    onClassMachineFilesMake,
    onClassMachineFilesUpload,
    onClassMachineFilesSendMd,
    onClassMachineModelAdd,
    onClassMachineModelDelete,
    onClassMachineModelSendRq
}) => {
    //Подключаемся к теме
    const theme = useTheme();

    //При нажатии на "Удалить" для карточки
    const handleCardDeleteClick = () => (onCardDelete ? onCardDelete(card.NRN) : null);

    //При нажатии на "Сформировать" для файлов данных
    const handleClassMachineFilesMakeClick = () => (onClassMachineFilesMake ? onClassMachineFilesMake(card.NRN, card.SUSERPROCS_DATA) : null);

    //При нажатии на "Загрузить на сервер" для файлов данных
    const handleClassMachineFilesUploadClick = () => (onClassMachineFilesUpload ? onClassMachineFilesUpload(card.NRN) : null);

    //При нажатии на "Загрузить на сервер" для файлов данных
    const handleClassMachineFilesSendMdClick = () => (onClassMachineFilesSendMd ? onClassMachineFilesSendMd(card.NRN) : null);

    //При нажатии на "Добавить" для моделей
    const handleClassMachineModelAddClick = () => (onClassMachineModelAdd ? onClassMachineModelAdd() : null);

    //При нажатии на "Удалить" для моделей
    const handleClassMachineModelDeleteClick = equipDSCMML => (onClassMachineModelDelete ? onClassMachineModelDelete(equipDSCMML) : null);

    //При нажатии на "Обучить" для моделей
    const handleClassMachineModelSendRqClick = equipDSCMML => (onClassMachineModelSendRq ? onClassMachineModelSendRq(equipDSCMML) : null);

    //Генерация содержимого
    return (
        <Box p={2} sx={STYLES.EQUIP_DSCM_CARD}>
            {card.NRN ? (
                <Stack spacing={2}>
                    <Stack spacing={2} direction={"row"} alignItems={"center"}>
                        <Typography variant="h5" component="div">
                            {card.SNAME}
                        </Typography>
                        <IconButton onClick={handleCardDeleteClick}>
                            <Icon>delete</Icon>
                        </IconButton>
                    </Stack>
                    <Card elevation={6}>
                        <CardContent>
                            <Stack spacing={2} direction={"row"} alignItems={"center"}>
                                <Typography variant="h6" component="div">
                                    Файлы данных
                                </Typography>
                                <Stack spacing={0} direction={"row"}>
                                    <Button onClick={handleClassMachineFilesMakeClick}>Сформировать</Button>
                                    <Button onClick={handleClassMachineFilesUploadClick}>Загрузить на сервер</Button>
                                    <Button onClick={handleClassMachineFilesSendMdClick}>Передать фреймворку</Button>
                                </Stack>
                            </Stack>
                            <P8PDataGrid
                                {...{ ...P8P_DATA_GRID_CONFIG_PROPS }}
                                containerComponentProps={{ sx: STYLES.FL_ML_TABLE, elevation: 0 }}
                                columnsDef={filesList.columnsDef}
                                rows={filesList.rows}
                                size={P8P_DATA_GRID_SIZE.SMALL}
                                morePages={false}
                                fixedHeader={true}
                                reloading={false}
                                headCellRender={classMachineCardTableHeadCellRender}
                                dataCellRender={prms => filesListDataCellRender({ ...prms, theme })}
                            />
                        </CardContent>
                    </Card>
                    <Card elevation={6}>
                        <CardContent>
                            <Stack spacing={2} direction={"row"} alignItems={"center"}>
                                <Typography variant="h6" component="div">
                                    Модели
                                </Typography>
                                <IconButton onClick={handleClassMachineModelAddClick}>
                                    <Icon>add</Icon>
                                </IconButton>
                            </Stack>
                            <P8PDataGrid
                                {...{ ...P8P_DATA_GRID_CONFIG_PROPS }}
                                containerComponentProps={{ sx: STYLES.FL_ML_TABLE, elevation: 0 }}
                                columnsDef={modelsList.columnsDef}
                                rows={modelsList.rows}
                                size={P8P_DATA_GRID_SIZE.SMALL}
                                morePages={false}
                                fixedHeader={true}
                                reloading={false}
                                headCellRender={classMachineCardTableHeadCellRender}
                                dataCellRender={prms =>
                                    modelsListDataCellRender({
                                        ...prms,
                                        theme,
                                        onDelete: handleClassMachineModelDeleteClick,
                                        onSendRq: handleClassMachineModelSendRqClick
                                    })
                                }
                            />
                        </CardContent>
                    </Card>
                </Stack>
            ) : null}
        </Box>
    );
};

//Контроль свойств - Карточка класса оборудования выборки данных
EquipDataSelectionClassMachineCard.propTypes = {
    card: PropTypes.object.isRequired,
    filesList: PropTypes.object.isRequired,
    modelsList: PropTypes.object.isRequired,
    onCardDelete: PropTypes.func,
    onClassMachineFilesMake: PropTypes.func,
    onClassMachineFilesUpload: PropTypes.func,
    onClassMachineFilesSendMd: PropTypes.func,
    onClassMachineModelAdd: PropTypes.func,
    onClassMachineModelDelete: PropTypes.func,
    onClassMachineModelSendRq: PropTypes.func
};

//----------------
//Интерфейс модуля
//----------------

export {
    EquipDataSelectionIU,
    EquipDataSelectionClassMachineIU,
    EquipDataSelectionClassMachineModelIU,
    EquipDataSelectionList,
    EquipDataSelectionClassMachineList,
    EquipDataSelectionClassMachineCard
};
