/*
    Парус 8 - Панели мониторинга - ТОиР - Прогнозирование технического состояния
    Закладка прогнозирования
*/

//---------------------
//Подключение библиотек
//---------------------

import React, { useState, useContext } from "react"; //Классы React
import PropTypes from "prop-types"; //Контроль свойств компонента
import { Box, Grid } from "@mui/material"; //Интерфейсные компоненты
import { RichTreeView } from "@mui/x-tree-view/RichTreeView"; //Дерево
import { MessagingСtx } from "../../context/messaging"; //Контекст сообщений
import { BackEndСtx } from "../../context/backend"; //Контекст взаимодействия с серверомs
import { ApplicationСtx } from "../../context/application"; //Контекст приложения
import { P8PDataGrid, P8P_DATA_GRID_SIZE } from "../../components/p8p_data_grid"; //Таблица данных
import { P8P_DATA_GRID_CONFIG_PROPS } from "../../config_wrapper"; //Подключение компонентов к настройкам приложения
import { APP_BAR_HEIGHT, TABS_HEIGHT, TABLE_FILTERS_HEIGHT, TABLE_MORE_HEIGHT, SCROLL_STYLES } from "./eqs_tech_cond_forecast_lyaout"; //Общие Вспомогательные компоненты и вёрстка
import { TechObjCard, eqConfigTechObjTableValueFormatter, eqConfigTechObjTableDataCellRender, TechObjMakeEqRpSheet } from "./forecast_tab_layout"; //Вспомогательные компоненты и вёрстка
import {
    useEqConfigTree,
    useEqConfigTechObjTable,
    useEqConfigTechObjCard,
    useTechObjModelsList,
    useTechObjForecastHistList,
    needLoadLevel
} from "./forecast_tab_hooks"; //Вспомогательные хуки

//---------
//Константы
//---------

//Начальное состояние флагов обновления
const REFRESH_INITIAL = {
    techObjForecastHistList: 0
};

//Начальное состояние диалогов
const DIALOGS_INITIAL = { makeEqRpSheet: false };

//Стили
const STYLES = {
    LEFT_SIDE_GRID: {},
    RIGHT_SIDE_GRID: {},
    TREE_BOX: {
        minHeight: `calc(100vh - ${APP_BAR_HEIGHT} - ${TABS_HEIGHT})`,
        maxHeight: `calc(100vh - ${APP_BAR_HEIGHT} - ${TABS_HEIGHT})`,
        overflowX: "hidden",
        overflowY: "auto",
        padding: "3px",
        ...SCROLL_STYLES
    },
    TECH_OBJ_TABLE: (morePages, filters) => ({
        height: `calc(100vh - ${APP_BAR_HEIGHT} - ${TABS_HEIGHT} - ${morePages ? TABLE_MORE_HEIGHT : "0px"} - ${
            filters ? TABLE_FILTERS_HEIGHT : "0px"
        })`,
        ...SCROLL_STYLES
    })
};

//Начальное состояние спецификации техничких объектов
const TECH_OBJ_SPEC_INIT = { parent: null, filters: [], orders: [], pageNumber: 1 };

//-----------
//Тело модуля
//-----------

//Закладка прогнозирования
const ForecastTab = ({ onGoToAdmin }) => {
    //Подключение к контексту взаимодействия с сервером
    const { executeStored } = useContext(BackEndСtx);

    //Подключение к контексту сообщений
    const { InlineMsgInfo } = useContext(MessagingСtx);

    //Подключение к контексту приложения
    const { pOnlineShowUnit } = useContext(ApplicationСtx);

    //Собственное состояние - текущая загружаемая ветка
    const [loadingTreeItem, setLoadingTreeItem] = useState(0);

    //Собственное состояние - спецификация технических объектов
    const [techObjSpec, setTechObjSpec] = useState({ ...TECH_OBJ_SPEC_INIT });

    //Собственное состояния - карточка технического объекта - 11984229
    const [techObjCardId, setTechObjCardId] = useState(null);

    //Собственное состояние - отображаемые диалоги
    const [dialogs, setDialogs] = useState(DIALOGS_INITIAL);

    //Собственное состояния - флаги перезагрузки
    const [refresh, setRefresh] = useState(REFRESH_INITIAL);

    //Загрузчик веток дерева
    const { eQconfigTree } = useEqConfigTree(loadingTreeItem);

    //Загрузчик технических объектов для выбранной ветки
    const { techObjsDataGrid, techObjsDataGridIsLoading } = useEqConfigTechObjTable(
        techObjSpec.parent,
        techObjSpec.filters,
        techObjSpec.orders,
        techObjSpec.pageNumber
    );

    //Загрузчик карточки выбранного технического объекта
    const { techObjCard, techObjCardIsLoading } = useEqConfigTechObjCard(techObjCardId);

    //Загрузчик моделей выбранного технического объекта
    const { techObjModelsList } = useTechObjModelsList(techObjCardId);

    //
    const { techObjForecastHistList } = useTechObjForecastHistList(techObjCardId, refresh.techObjForecastHistList);

    //Обработка развёртывания/свёртывания уровня дерева
    const handleTreeItemExpansionToggle = (event, itemId, isExpanded) =>
        isExpanded && needLoadLevel(eQconfigTree, itemId) ? setLoadingTreeItem(parseInt(itemId)) : null;

    //Обработка фокусировки на элементе дерева
    const handleTreeItemFocus = (event, itemId) => setTechObjSpec({ ...TECH_OBJ_SPEC_INIT, parent: parseInt(itemId) });

    //При изменении состояния фильтров спецификации технических объектов
    const handleTechObjSpecFilterChanged = ({ filters }) => setTechObjSpec(pv => ({ ...pv, filters: [...filters], pageNumber: 1 }));

    //При изменении состояния сортировки спецификации технических объектов
    const handleTechObjSpecOrderChanged = ({ orders }) => setTechObjSpec(pv => ({ ...pv, orders: [...orders], pageNumber: 1 }));

    //При изменении количества отображаемых страниц спецификации технических объектов
    const handleTechObjSpecPagesCountChanged = () => setTechObjSpec(pv => ({ ...pv, pageNumber: pv.pageNumber + 1 }));

    //При выполнении действия в колоке "Модели" таблицы технических объектов
    const handleCMMLStatusClick = (event, row) => {
        if (row.NCMML_STATUS == 0) {
            if (onGoToAdmin) onGoToAdmin();
        } else setTechObjCardId(parseInt(row.NRN));
    };

    //При переходе к закладке администрирования из карточки технического объекта
    const handleTechObjeCardGoToModel = model => {
        handleTechObjeCardClose();
        if (onGoToAdmin) onGoToAdmin(model.NEQUIPDS, model.NEQUIPDSCM);
    };

    //При запросе прогноза по модели из карточки технического объекта
    const handleTechObjeCardGetPrediction = async model => {
        await executeStored({ stored: "UDO_PKG_EQUIPTCF.EQCONFIG_THOBJ_FORECAST", args: { NEQCONFIG: techObjCardId, NEQUIPDSCMML: model.NRN } });
        setRefresh(pv => ({ ...pv, techObjForecastHistList: pv.techObjForecastHistList + 1 }));
    };

    //При закрытии карточки технического объекта
    const handleTechObjeCardClose = () => setTechObjCardId(null);

    //При нажатии на "Ремонтировать" в карточке технического объекта
    const handleTechObjeCardMakeEqRpSheet = () => setDialogs(pv => ({ ...pv, makeEqRpSheet: true }));

    //При нажатии "ОК" в диалоге формирования ремонтной ведомости
    const handleTechObjMakeEqRpSheetOk = async values => {
        const data = await executeStored({
            stored: "UDO_PKG_EQUIPTCF.EQCONFIG_THOBJ_MAKE_EQRPSHEET",
            args: {
                NEQCONFIG: techObjCardId,
                SACATALOG: values.acatalog,
                SEQTECSRVKIND: values.eQTecSrvKind,
                DPLANDATE_FROM: new Date(values.planDateFrom)
            }
        });
        setDialogs(pv => ({ ...pv, makeEqRpSheet: false }));
        pOnlineShowUnit({ unitCode: "EquipRepairSheets", showMethod: "main_selected", inputParameters: [{ name: "in_Ident", value: data.NIDENT }] });
    };

    //При нажитии "Отмена" в диалоге формирования ремонтной ведомости
    const handleTechObjMakeEqRpSheetCancel = () => setDialogs(pv => ({ ...pv, makeEqRpSheet: false }));

    //Текст при отсутствии данных в списке технических объектов
    const noDataFoundText =
        techObjsDataGridIsLoading || !techObjsDataGrid.init
            ? "Минуточку..."
            : techObjSpec.filters.length > 0
            ? "Нет данных, соответствующих условиям отбора"
            : "Выбранный уровень структуры не содержит данных о технических объектах";

    //Генерация содержимого
    return (
        <div>
            {dialogs.makeEqRpSheet ? <TechObjMakeEqRpSheet onCancel={handleTechObjMakeEqRpSheetCancel} onOk={handleTechObjMakeEqRpSheetOk} /> : null}
            <Grid container>
                <Grid item xs={3} sx={STYLES.LEFT_SIDE_GRID}>
                    <Box sx={STYLES.TREE_BOX}>
                        <RichTreeView items={eQconfigTree} onItemExpansionToggle={handleTreeItemExpansionToggle} onItemFocus={handleTreeItemFocus} />
                    </Box>
                </Grid>
                <Grid item xs={9} sx={STYLES.RIGHT_SIDE_GRID}>
                    {techObjCardId && !techObjCardIsLoading && techObjCard?.SCODE ? (
                        <TechObjCard
                            cardData={techObjCard}
                            modelsList={techObjModelsList}
                            forecastHistList={techObjForecastHistList}
                            onClose={handleTechObjeCardClose}
                            onGoToModel={handleTechObjeCardGoToModel}
                            onGetPrediction={handleTechObjeCardGetPrediction}
                            onMakeEqRpSheet={handleTechObjeCardMakeEqRpSheet}
                        />
                    ) : null}
                    {techObjsDataGrid.init ? (
                        <P8PDataGrid
                            {...{ ...P8P_DATA_GRID_CONFIG_PROPS, noDataFoundText }}
                            containerComponentProps={{
                                sx: STYLES.TECH_OBJ_TABLE(techObjsDataGrid.morePages, techObjSpec.filters.length > 0),
                                elevation: 0
                            }}
                            filtersInitial={techObjSpec.filters}
                            columnsDef={techObjsDataGrid.columnsDef}
                            rows={techObjsDataGrid.rows}
                            size={P8P_DATA_GRID_SIZE.SMALL}
                            morePages={techObjsDataGrid.morePages}
                            fixedHeader={true}
                            reloading={false}
                            valueFormatter={eqConfigTechObjTableValueFormatter}
                            dataCellRender={prms => eqConfigTechObjTableDataCellRender({ ...prms, onCMMLStatus: handleCMMLStatusClick })}
                            onFilterChanged={handleTechObjSpecFilterChanged}
                            onOrderChanged={handleTechObjSpecOrderChanged}
                            onPagesCountChanged={handleTechObjSpecPagesCountChanged}
                        />
                    ) : (
                        <InlineMsgInfo okBtn={false} text={"Укажите элемент структуры состава оборудования для отображения данных"} />
                    )}
                </Grid>
            </Grid>
        </div>
    );
};

//Контроль свойств - Закладка прогнозирования
ForecastTab.propTypes = {
    onGoToAdmin: PropTypes.func
};

//----------------
//Интерфейс модуля
//----------------

export { ForecastTab };
