/*
    Парус 8 - Панели мониторинга - ТОиР - Прогнозирование технического состояния
    Закладка прогнозирования: дополнительная разметка и вёрстка клиентских элементов
*/

//---------------------
//Подключение библиотек
//---------------------

import React, { useState, useEffect, useContext } from "react"; //Классы React
import PropTypes from "prop-types"; //Контроль свойств компонента
import { Stack, Card, CardContent, Typography, Button, Link, Dialog, DialogTitle, DialogContent, DialogActions } from "@mui/material"; //Интерфейсные компоненты
import { useTheme } from "@mui/material/styles"; //Темы оформления
import { ApplicationСtx } from "../../context/application"; //Контекст приложения
import { BUTTONS } from "../../../app.text"; //Текстовые ресурсы и константы
import { P8PDataGrid, P8P_DATA_GRID_SIZE } from "../../components/p8p_data_grid"; //Таблица данных
import { P8P_DATA_GRID_CONFIG_PROPS } from "../../config_wrapper"; //Подключение компонентов к настройкам приложения
import { formatDateRF, xml2JSON } from "../../core/utils"; //Вспомогательные функции
import { SCROLL_STYLES, formatModelStateValue, IUDFormTextField } from "./eqs_tech_cond_forecast_lyaout"; //Общие вспомогательные компоненты и вёрстка
import { P8PChart } from "../../components/p8p_chart"; //График

//---------
//Константы
//---------

//Стили
const STYLES = {
    TECH_OBJ_CARD_DIALOG: { maxWidth: "800px" },
    TECH_OBJ_CARD_ML_TABLE: {
        maxHeight: "300px",
        ...SCROLL_STYLES
    },
    TECH_OBJ_CARD_FORECAST_TABLE: {
        maxHeight: "300px",
        ...SCROLL_STYLES
    },
    TECH_OBJ_FORECAST_DETAIL_DIALOG: { maxWidth: "600px" },
    TECH_OBJ_FORECAST_DETAIL_CHART: { width: "550px", display: "flex", justifyContent: "center" }
};

//------------------------------------
//Вспомогательные функции и компоненты
//------------------------------------

//Форматирование ячеек заголовка таблицы моделей единицы оборудования
const techObjCardModelsTableHeadCellRender = ({ columnDef }) => {
    switch (columnDef.name) {
        case "NSTATUS":
            return {
                stackProps: { justifyContent: "left" },
                cellProps: { align: "left" }
            };
        case "SACTIONS":
            return { data: " " };
    }
};

//Форматирование колонок таблицы моделей единицы оборудования
const techObjCardModelsTableDataCellRender = ({ row, columnDef, theme, onGoToModel, onGetPrediction }) => {
    switch (columnDef.name) {
        case "SNEQUIPDS":
        case "STASK":
        case "NPRECISION_P":
        case "NPRECISION_F":
            return {
                data: (
                    <Link component="button" variant="body2" align="left" underline="hover" onClick={() => onGoToModel(row)}>
                        {row[columnDef.name]}
                    </Link>
                )
            };
        case "NSTATUS":
            return {
                cellProps: { align: "left" },
                data: formatModelStateValue(theme, row.NSTATUS, row.SERR)
            };
        case "SACTIONS":
            return {
                data:
                    row.NSTATUS == 2 ? (
                        <Button variant="outlined" color="success" onClick={() => onGetPrediction(row)}>
                            Прогноз
                        </Button>
                    ) : null
            };
    }
};

//Форматирование колонок таблицы истории прогнозов класса оборудования выборки данных
const techObjCardForecastListTableDataCellRender = ({ row, columnDef, onShowForecastDetail }) => {
    switch (columnDef.name) {
        case "NFORECAST":
            return {
                data: (
                    <Button
                        variant="outlined"
                        color={row.NFORECAST < 30 ? "success" : row.NFORECAST >= 30 && row.NFORECAST < 60 ? "warning" : "error"}
                        onClick={() => onShowForecastDetail(row)}
                    >
                        {`${row.NFORECAST}%`}
                    </Button>
                )
            };
    }
};

//Детали прогноза
const ForecastDetail = ({ date, forecast, onClose }) => {
    //Собственное состояние - график
    const [chart, setChart] = useState({ loaded: false, labels: [], datasets: [] });

    //При подключении к странице
    useEffect(() => {
        const loadChart = async () => {
            const chart = await xml2JSON({ xmlDoc: forecast });
            console.log(chart);
            setChart(pv => ({ ...pv, loaded: true, ...chart.XDATA.XCHART }));
        };
        loadChart();
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);

    //Генерация содержимого
    return (
        <Dialog open={true} onClose={() => (onClose ? onClose() : null)} {...STYLES.TECH_OBJ_FORECAST_DETAIL_DIALOG}>
            <DialogTitle>{`Детали прогноза от ${date}`}</DialogTitle>
            <DialogContent>{chart.loaded ? <P8PChart {...chart} style={STYLES.TECH_OBJ_FORECAST_DETAIL_CHART} /> : null}</DialogContent>
            <DialogActions>
                <Button onClick={() => (onClose ? onClose() : null)}>{BUTTONS.CLOSE}</Button>
            </DialogActions>
        </Dialog>
    );
};

//Контроль свойств - Детали прогноза
ForecastDetail.propTypes = {
    date: PropTypes.string.isRequired,
    forecast: PropTypes.string.isRequired,
    onClose: PropTypes.func
};

//-----------
//Тело модуля
//-----------

//Карточка технического объекта
const TechObjCard = ({ cardData, modelsList, forecastHistList, onClose, onGoToModel, onGetPrediction, onMakeEqRpSheet }) => {
    //Подключаемся к теме
    const theme = useTheme();

    //Собственное состояние
    const [state, setState] = useState({ forecastDetail: null, forecastDate: null });

    //При нажатии на кнопку закрытия карточки
    const handleClose = () => (onClose ? onClose() : null);

    //При нажатии на переход к модели
    const handleGoToModelClick = model => (onGoToModel ? onGoToModel(model) : null);

    //При нажатии на запрос предсказания
    const handleGetPredictionClick = model => (onGetPrediction ? onGetPrediction(model) : null);

    //При нажатии на формирование ремонтной ведомости
    const handleMakeEqRpSheetClick = () => (onMakeEqRpSheet ? onMakeEqRpSheet() : null);

    //При нажатии на отображение деталей прогноза
    const handleShowForecastDetailClick = modelHist => {
        setState(pv => ({ ...pv, forecastDetail: modelHist.SFORECAST, forecastDate: modelHist.SRQ_DATE }));
    };

    //При нажатии на закрытие деталей прогноза
    const handleCloseForecastDetailClick = () => setState(pv => ({ ...pv, forecastDetail: null, forecastDate: null }));

    //Генерация содержимого
    return (
        <>
            {state.forecastDetail ? (
                <ForecastDetail date={state.forecastDate} forecast={state.forecastDetail} onClose={handleCloseForecastDetailClick} />
            ) : null}
            <Dialog open={true} onClose={handleClose} {...STYLES.TECH_OBJ_CARD_DIALOG}>
                <DialogTitle>{cardData.SCODE}</DialogTitle>
                <DialogContent>
                    <Typography variant="h6">{cardData.SNAME}</Typography>
                    <Stack spacing={2}>
                        <Typography color="text.secondary" variant="caption" gutterBottom>
                            {`${formatDateRF(cardData.DOPER_DATE)}, ${cardData.SEQOBJKIND}`}
                        </Typography>
                        {modelsList.init ? (
                            <Card elevation={6}>
                                <CardContent>
                                    <Typography variant="h6" component="div">
                                        Модели
                                    </Typography>
                                    <P8PDataGrid
                                        {...{ ...P8P_DATA_GRID_CONFIG_PROPS }}
                                        containerComponentProps={{ sx: STYLES.TECH_OBJ_CARD_ML_TABLE, elevation: 0 }}
                                        columnsDef={modelsList.columnsDef}
                                        rows={modelsList.rows}
                                        size={P8P_DATA_GRID_SIZE.SMALL}
                                        morePages={false}
                                        fixedHeader={true}
                                        reloading={false}
                                        headCellRender={techObjCardModelsTableHeadCellRender}
                                        dataCellRender={prms =>
                                            techObjCardModelsTableDataCellRender({
                                                ...prms,
                                                theme,
                                                onGoToModel: handleGoToModelClick,
                                                onGetPrediction: handleGetPredictionClick
                                            })
                                        }
                                    />
                                </CardContent>
                            </Card>
                        ) : null}
                        {forecastHistList.init ? (
                            <Card elevation={6}>
                                <CardContent>
                                    <Typography variant="h6" component="div">
                                        Прогнозы
                                    </Typography>
                                    <P8PDataGrid
                                        {...{ ...P8P_DATA_GRID_CONFIG_PROPS }}
                                        containerComponentProps={{ sx: STYLES.TECH_OBJ_CARD_FORECAST_TABLE, elevation: 0 }}
                                        columnsDef={forecastHistList.columnsDef}
                                        rows={forecastHistList.rows}
                                        size={P8P_DATA_GRID_SIZE.SMALL}
                                        morePages={false}
                                        fixedHeader={true}
                                        reloading={false}
                                        dataCellRender={prms =>
                                            techObjCardForecastListTableDataCellRender({
                                                ...prms,
                                                theme,
                                                onShowForecastDetail: handleShowForecastDetailClick
                                            })
                                        }
                                    />
                                </CardContent>
                            </Card>
                        ) : null}
                    </Stack>
                </DialogContent>
                <DialogActions>
                    <Button onClick={handleMakeEqRpSheetClick} color={"error"}>
                        Ремонтировать
                    </Button>
                    <Button onClick={handleClose}>{BUTTONS.CLOSE}</Button>
                </DialogActions>
            </Dialog>
        </>
    );
};

//Контроль свойств - Карточка технического объекта
TechObjCard.propTypes = {
    cardData: PropTypes.object.isRequired,
    modelsList: PropTypes.object.isRequired,
    forecastHistList: PropTypes.object.isRequired,
    onClose: PropTypes.func,
    onGoToModel: PropTypes.func,
    onGetPrediction: PropTypes.func,
    onMakeEqRpSheet: PropTypes.func
};

//Формирование значения для колонки "Медель" таблицы технических объектов
const formatTechObjCMMLStatusValue = value => (value == 0 ? "Обучить" : value == 1 ? "Обучается" : "Прогнозировать");

//Формирование цвета для колонки "Медель" таблицы технических объектов
const formatTechObjCMMLStatusColor = value => (value == 0 ? "primary" : value == 1 ? "warning" : "success");

//Форматирование значений колонок таблицы технических объектов
const eqConfigTechObjTableValueFormatter = ({ value, columnDef }) => {
    switch (columnDef.name) {
        case "DOPER_DATE":
            return formatDateRF(value);
        case "NCMML_STATUS":
            return formatTechObjCMMLStatusValue(value);
    }
    return value;
};

//Форматирование колонок таблицы технических объектов
const eqConfigTechObjTableDataCellRender = ({ row, columnDef, onCMMLStatus }) => {
    switch (columnDef.name) {
        case "NCMML_STATUS":
            return {
                data: (
                    <Button
                        variant="outlined"
                        color={formatTechObjCMMLStatusColor(row[columnDef.name])}
                        onClick={event => (onCMMLStatus ? onCMMLStatus(event, row) : null)}
                    >
                        {formatTechObjCMMLStatusValue(row[columnDef.name])}
                    </Button>
                )
            };
    }
};

//Диалог формирование ремонтной ведомости
const TechObjMakeEqRpSheet = ({ onOk, onCancel }) => {
    //Подключение к контексту приложения
    const { pOnlineShowDictionary } = useContext(ApplicationСtx);

    //Собственное состояние - значения формы
    const [values, setValues] = useState({
        acatalog: "",
        eQTecSrvKind: "",
        planDateFrom: ""
    });

    //Отработка воода значения в форму
    const handleValueChanged = (name, value) => setValues(pv => ({ ...pv, [name]: value }));

    //Выбор каталога размещения ремонтной ведомости
    const selectEquipRepairSheetsCatalog = (name, callBack) => {
        pOnlineShowDictionary({
            unitCode: "EquipRepairSheets",
            showMethod: "catalog",
            callBack: res => callBack(res.success ? [{ name, value: res.outParameters.ctlgname }] : null)
        });
    };

    //Выбор вида технического обслуживания
    const selectEquipTechServiceKinds = (name, callBack) => {
        pOnlineShowDictionary({
            unitCode: "EquipTechServiceKinds",
            callBack: res => callBack(res.success ? [{ name, value: res.outParameters.out_CODE }] : null)
        });
    };

    //Генерация содержимого
    return (
        <Dialog open={true} onClose={() => (onOk ? onCancel() : null)}>
            <DialogTitle>Формирование ремонтной ведомости</DialogTitle>
            <DialogContent>
                <IUDFormTextField
                    elementCode={"acatalog"}
                    elementValue={values.acatalog}
                    labelText={"Каталог размещения"}
                    onChange={handleValueChanged}
                    dictionary={callBack => selectEquipRepairSheetsCatalog("acatalog", callBack)}
                />
                <IUDFormTextField
                    elementCode={"eQTecSrvKind"}
                    elementValue={values.eQTecSrvKind}
                    labelText={"Вид технического обслуживания"}
                    onChange={handleValueChanged}
                    dictionary={callBack => selectEquipTechServiceKinds("eQTecSrvKind", callBack)}
                />
                <IUDFormTextField
                    elementCode={"planDateFrom"}
                    elementValue={values.planDateFrom}
                    labelText={"Плановая дата ремонта"}
                    onChange={handleValueChanged}
                    type={"date"}
                />
            </DialogContent>
            <DialogActions>
                <Button onClick={() => (onOk ? onOk(values) : null)}>{BUTTONS.OK}</Button>
                <Button onClick={() => (onOk ? onCancel() : null)}>{BUTTONS.CANCEL}</Button>
            </DialogActions>
        </Dialog>
    );
};

//Контроль свойств - Диалог формирование ремонтной ведомости
TechObjMakeEqRpSheet.propTypes = {
    onOk: PropTypes.func,
    onCancel: PropTypes.func
};

//----------------
//Интерфейс модуля
//----------------

export { TechObjCard, eqConfigTechObjTableValueFormatter, eqConfigTechObjTableDataCellRender, TechObjMakeEqRpSheet };
